@@ -25,18 +25,15 @@
 #include "system.h"
 #include "config.h"
 #ifdef ENABLE_XKB
-#include <X11/XKBlib.h>
+	#include <X11/XKBlib.h>
 #endif
 #include <gtk/gtk.h>
 #include "status.h"
 #include "trayicon.h"
 #ifdef ENABLE_RAMBLE
-#include "ramble.h"
-gboolean ramble_add(struct ramble *ramble, GdkWindow *window, gint x, gint y, struct key *k);
-#endif
-#ifdef APPLET
-#include <panel-applet.h>
+	#include "ramble.h"
 #endif
+#include "service.h"
 
 /* There is one florence structure which contains all global data in florence.c */
 struct florence {
@@ -56,14 +53,11 @@ struct florence {
 #ifdef ENABLE_AT_SPI
 	Accessible *obj; /* editable object being selected */
 #endif
+	struct service *service; /* dbus service object */
 };
 
 /* create a new instance of florence. */
-#ifdef APPLET
-struct florence *flo_new(gboolean gnome, const gchar *focus_back, PanelApplet *applet);
-#else
 struct florence *flo_new(gboolean gnome, const gchar *focus_back);
-#endif
 /* liberate all the memory used by florence */
 void flo_free(struct florence *florence);
 
