@@ -1,7 +1,7 @@
 /* 
    Florence - Florence is a simple virtual keyboard for Gnome.
 
-   Copyright (C) 2008, 2009, 2010 François Agrech
+   Copyright (C) 2012 François Agrech
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,110 +28,22 @@
 #define STATUS_EVENTCHECK_INTERVAL 100
 /* animate keyboard every 1/50th of a second */
 #define STATUS_ANIMATION_INTERVAL 20
-
-/* action for the fsm table */
-typedef void (*status_action) (struct status *, struct key *, enum status_event);
-
-/* what to do when state changes */
-struct status_change {
-	enum key_state new_state; /* new state after the change */
-	status_action *actions; /* actions to do to switch state (NULL terminated list) */
-};
-
-/* all FSM actions */
-void status_send (struct status *, struct key *, enum status_event);
-void status_send_latched (struct status *, struct key *, enum status_event);
-void status_latch (struct status *, struct key *, enum status_event);
-void status_unlatch (struct status *, struct key *, enum status_event);
-void status_unlatch_all (struct status *, struct key *, enum status_event);
-void status_lock (struct status *, struct key *, enum status_event);
-void status_unlock (struct status *, struct key *, enum status_event);
-void status_update_view (struct status *, struct key *, enum status_event);
-void status_error (struct status *, struct key *, enum status_event);
-
-/* FSM action groups */
-static status_action status_action_s[]={ status_send, NULL };
-static status_action status_action_la[]={ status_latch, status_update_view, NULL };
-static status_action status_action_ulalo[]={ status_unlatch, status_lock, status_update_view, NULL };
-static status_action status_action_lo[]={ status_lock, status_update_view, NULL };
-static status_action status_action_ul[]={ status_unlock, status_update_view, NULL };
-static status_action status_action_slas[]={ status_send_latched, status_send, NULL };
-static status_action status_action_sslaulaa[]={ status_send, status_send_latched, status_unlatch_all, status_update_view, NULL };
-static status_action status_action_uv[]={ status_update_view, NULL };
-static status_action status_action_err[]={ status_error, NULL };
-
-/* status FSM - simulate sticky keys
- * changes by type, event and state */
-static struct status_change status_fsm[STATUS_EVENT_NUM][STATUS_KEY_TYPE_NUM][KEY_STATE_NUM]={
-	{ /* PRESS event */
-		{ /* NORMAL key */
-			{ KEY_PRESSED, NULL }, /* PRESSED state */
-			{ KEY_RELEASED, status_action_slas }, /* RELEASED state */
-		}, { /* MODIFIER key */
-			{ KEY_RELEASED, status_action_err }, /* PRESSED state */
-			{ KEY_LATCHED, status_action_la }, /* RELEASED state */
-			{ KEY_RELEASED, status_action_ul }, /* LOCKED state */
-			{ KEY_LOCKED, status_action_ulalo } /* LATHCED state */
-		}, { /* LOCKER key */
-			{ KEY_RELEASED, status_action_err }, /* PRESSED state */
-			{ KEY_RELEASED, status_action_s }, /* RELEASED state */
-			{ KEY_LOCKED, status_action_s } /* LOCKED state */
-		}
-	}, { /* RELEASE event */
-		{ /* NORMAL key */
-			{ KEY_PRESSED, status_action_sslaulaa }, /* PRESSED state */
-			{ KEY_RELEASED, NULL }, /* RELEASED state */
-		}, { /* MODIFIER key */
-			{ KEY_RELEASED, status_action_err }, /* PRESSED state */
-			{ KEY_RELEASED, NULL }, /* RELEASED state */
-			{ KEY_LOCKED, NULL }, /* LOCKED state */
-			{ KEY_LATCHED, NULL } /* LATHCED state */
-		}, { /* LOCKER key */
-			{ KEY_RELEASED, status_action_err }, /* PRESSED state */
-			{ KEY_RELEASED, status_action_s }, /* RELEASED state */
-			{ KEY_LOCKED, status_action_s } /* LOCKED state */
-		}
-	}, { /* PRESSED event */
-		{ /* NORMAL key */
-			{ KEY_PRESSED, NULL }, /* PRESSED state */
-			{ KEY_PRESSED, status_action_uv } /* RELEASED state */
-		}, { /* MODIFIER key */
-			{ KEY_PRESSED, status_action_err }, /* PRESSED state */
-			{ KEY_RELEASED, NULL }, /* RELEASED state */
-			{ KEY_LOCKED, NULL }, /* LOCKED state */
-			{ KEY_LATCHED, NULL } /* LATHCED state */
-		}, { /* LOCKER key */
-			{ KEY_RELEASED, status_action_err }, /* PRESSED state */
-			{ KEY_LOCKED, status_action_lo }, /* RELEASED state */
-			{ KEY_RELEASED, status_action_ul } /* LOCKED state */
-		}
-	}, { /* RELEASED event */
-		{ /* NORMAL key */
-			{ KEY_RELEASED, status_action_uv }, /* PRESSED state */
-			{ KEY_RELEASED, NULL } /* RELEASED state */
-		}, { /* MODIFIER key */
-			{ KEY_RELEASED, status_action_err }, /* PRESSED state */
-			{ KEY_RELEASED, NULL }, /* RELEASED state */
-			{ KEY_LOCKED, NULL }, /* LOCKED state */
-			{ KEY_LATCHED, NULL } /* LATHCED state */
-		}, { /* LOCKER key */
-			{ KEY_RELEASED, status_action_err }, /* PRESSED state */
-			{ KEY_RELEASED, NULL }, /* RELEASED state */
-			{ KEY_LOCKED, NULL } /* LOCKED state */
-		}
-	}
-};
+/* show visual effect for touched keys for 200ms */
+#define STATUS_TOUCH_TIMEOUT 200
 
 /* handle X11 errors */
 int status_error_handler(Display *my_dpy, XErrorEvent *event)
 {
+	START_FUNC
 	flo_warn(_("Unable to focus window."));
+	END_FUNC
 	return 0;
 }
 
 /* switch focus to focus window */
 void status_focus_window(struct status *status)
 {
+	START_FUNC
 	int (*old_handler)(Display *, XErrorEvent *);
 	if (status->w_focus) {
 		old_handler=XSetErrorHandler(status_error_handler);
@@ -140,40 +52,22 @@ void status_focus_window(struct status *status)
 		XSync(gdk_x11_get_default_xdisplay(), FALSE);
 		XSetErrorHandler(old_handler);
 	}
-}
-
-/* process the fsm actions and state change */
-void status_fsm_process(struct status *status, struct key *key, enum status_event event)
-{
-	enum key_state state;
-	enum status_key_type type;
-	guint idx;
-	if (key) {
-		state=key->state;
-		type=(key_get_modifier(key)?(key_is_locker(key)?
-			STATUS_KEY_LOCKER:STATUS_KEY_MODIFIER):STATUS_KEY_NORMAL);
-		/* switch state */
-		key_state_set(key, status_fsm[event][type][state].new_state);
-		/* execute actions */
-		status_focus_window(status);
-		flo_debug(_("Event %d received :type %d. Switching from state %d to %d"),
-			event, type, state, status_fsm[event][type][state].new_state);
-		if (status_fsm[event][type][state].actions)
-			for (idx=0;status_fsm[event][type][state].actions[idx];idx++)
-				status_fsm[event][type][state].actions[idx](status, key, event);
-	}
+	END_FUNC
 }
 
 /* update the global modifier mask */
 void status_globalmod_set(struct status *status, GdkModifierType mod)
 {
+	START_FUNC
 	status->globalmod|=mod;
+	END_FUNC
 }
 
 #ifdef ENABLE_XRECORD
 /* Called when a record event is received from XRecord */
 void status_record_event (XPointer priv, XRecordInterceptData *hook)
 {
+	START_FUNC
 	xEvent *event;
 	struct status *status=(struct status *)priv;
 	struct key *key;
@@ -181,25 +75,29 @@ void status_record_event (XPointer priv, XRecordInterceptData *hook)
 		event=(xEvent *)hook->data;
 		if ((key=status->keys[event->u.u.detail])) {
 			if (event->u.u.type==KeyPress)
-				status_fsm_process(status, key, STATUS_PRESSED);
+				fsm_process(status, key, FSM_PRESSED);
 			else if (event->u.u.type==KeyRelease)
-				status_fsm_process(status, key, STATUS_RELEASED);
+				fsm_process(status, key, FSM_RELEASED);
 		}
 	}
 	if (hook) XRecordFreeData(hook);
+	END_FUNC
 }
 
 /* Process record events (every 1/10th of a second) */
 gboolean status_record_process (gpointer data)
 {
+	START_FUNC
 	struct status *status=(struct status *)data;
 	XRecordProcessReplies(status->data_disp);
+	END_FUNC
 	return TRUE;
 }
 
 /* Record keyboard events */
 gpointer status_record_start (gpointer data)
 {
+	START_FUNC
 	struct status *status=(struct status *)data;
 	int major, minor;
 	XRecordRange *range;
@@ -224,12 +122,14 @@ gpointer status_record_start (gpointer data)
 	}
 	else flo_warn(_("No XRecord extension found"));
 	if (!status->RecordContext) flo_warn(_("Keyboard synchronization is disabled."));
+	END_FUNC
 	return data;
 }
 
 /* Stop recording keyboard events */
 void status_record_stop (struct status *status)
 {
+	START_FUNC
 	if (status->RecordContext) {
 		XRecordDisableContext(status->data_disp, status->RecordContext);
 		XRecordFreeContext(status->data_disp, status->RecordContext);
@@ -240,32 +140,46 @@ void status_record_stop (struct status *status)
 		/* XCloseDisplay(status->data_disp); */
 		status->data_disp=NULL;
 	}
+	END_FUNC
 }
 
 /* Add keys to the keycode indexed keys */
 void status_keys_add(struct status *status, GSList *keys)
 {
+	START_FUNC
 	GSList *list=keys;
 	struct key *key;
+	struct key_mod *mod;
+	struct key_code *code;
 	while (list) {
-		key=(struct key *)list->data;
-		if (((struct key_mod *)key->mods)->type==KEY_CODE) status->keys[key->code]=key;
+		key=(struct key *)(list->data);
+		mod=(struct key_mod *)(key->mods->data);
+		if (mod->type==KEY_CODE) {
+			code=(struct key_code *)(mod->data);
+			status->keys[code->code]=key;
+		}
 		list=list->next;
 	}
+	END_FUNC
 }
 #endif
 
 /* update the focus key */
 void status_focus_set(struct status *status, struct key *focus)
 {
-	view_update(status->view, status->focus, FALSE);
+	START_FUNC
+	struct key *old = status->focus;
 	status->focus=focus;
+	view_update(status->view, old, FALSE);
 	view_update(status->view, status->focus, FALSE);
+	END_FUNC
 }
 
 /* return the focus key */
 struct key *status_focus_get(struct status *status)
 {
+	START_FUNC
+	END_FUNC
 	return status->focus;
 }
 
@@ -277,14 +191,29 @@ struct key *status_pressed_get(struct status *status) { return status->pressed;
  * WARNING: not multi-touch safe! */
 void status_pressed_set(struct status *status, struct key *pressed)
 {
-	enum status_event event;
+	START_FUNC
+	enum fsm_event event;
+	gboolean touch=(status_im_get(status)==STATUS_IM_TOUCH);
+
 	/* find actions in fsm table */
 	if (pressed) {
-		event=STATUS_PRESS;
-		status->pressed=pressed;
-	} else event=STATUS_RELEASE;
-	status_fsm_process(status, status->pressed, event);
-	status->pressed=pressed;
+		event=FSM_PRESS;
+		if ((!touch) || (key_get_action(pressed, status)==KEY_MOVE)) status->pressed=pressed;
+	} else {
+		event=FSM_RELEASE;
+		if (touch && !((status->pressed)&&(key_get_action(status->pressed, status)==KEY_MOVE))) {
+			if (status->pressed && (!key_get_modifier(status->pressed))) {
+				key_state_set(status->pressed, KEY_RELEASED);
+				view_update(status->view, status->pressed, FALSE);
+			}
+			status->pressed=status->focus;
+		}
+	}
+
+	fsm_process(status, status->pressed, event);
+	if (touch) { if (status->pressed && key_get_modifier(status->pressed)) status->pressed=NULL; }
+	else status->pressed=pressed;
+	END_FUNC
 }
 
 /****************************/
@@ -292,46 +221,87 @@ void status_pressed_set(struct status *status, struct key *pressed)
 /****************************/
 
 /* update the view according to the change */
-void status_update_view (struct status *status, struct key *key, enum status_event event)
+void status_update_view (struct status *status, struct key *key)
 {
-	/* update view */
+	START_FUNC
 	if (status->view) view_update(status->view, key, key_get_modifier(key));
+	END_FUNC
 }
 
-/* send the event: press or release the key */
-void status_send (struct status *status, struct key *key, enum status_event event)
+/* update the key */
+void status_update_key (struct status *status, struct key *key)
 {
-	flo_debug(_("sending event %d"), event);
-	switch(event) {
-		case STATUS_PRESS:
-			key_press(key, status);
+	START_FUNC
+	if (!key_get_modifier(key)) {
+		if (key->state==KEY_PRESSED) status->pressed=key;
+		else if ((key->state==KEY_RELEASED) && (status->pressed==key)) status->pressed=NULL;
+	}
+	if (status->view) view_update(status->view, key, FALSE);
+	END_FUNC
+}
+
+/* triggered after 200ms when a key has been touched. */
+gboolean status_touch_timer(gpointer data)
+{
+	START_FUNC
+	struct status *status=(struct status *)data;
+	if (status->pressed) {
+		key_state_set(status->pressed, KEY_RELEASED);
+		view_update(status->view, status->pressed, FALSE);
+		status->pressed=NULL;
+	}
+	status->touch_id=0;
+	END_FUNC
+	return FALSE;
+}
+
+/* send the press event */
+void status_press (struct status *status, struct key *key)
+{
+	START_FUNC
+	flo_debug(TRACE_DEBUG, _("sending press event"));
+	key_press(key, status);
 #ifdef ENABLE_XRECORD
-			if (key->actions)
+	if (((struct key_mod *)(key->mods->data))->type==KEY_ACTION)
 #endif
-			status_fsm_process(status, key, STATUS_PRESSED);
-			break;
-		case STATUS_RELEASE:
-			key_release(key, status);
+	fsm_process(status, key, FSM_PRESSED);
 #ifdef ENABLE_XRECORD
-			if (key->actions)
+	status_record_process(status);
 #endif
-			status_fsm_process(status, key, STATUS_RELEASED);
-			break;
-		default: flo_warn(_("Unknown event type received: %d"), event); break;
+	END_FUNC
+}
+
+/* send the release event */
+void status_release (struct status *status, struct key *key)
+{
+	START_FUNC
+	flo_debug(TRACE_DEBUG, _("sending release event"));
+	key_release(key, status);
+#ifdef ENABLE_XRECORD
+	if (((struct key_mod *)(key->mods->data))->type==KEY_ACTION)
+#endif
+	fsm_process(status, key, FSM_RELEASED);
+	if (status_im_get(status)==STATUS_IM_TOUCH && (!key_get_modifier(key))) {
+		key_state_set(key, KEY_PRESSED);
+		view_update(status->view, key, FALSE);
+		if (status->touch_id) g_source_remove(status->touch_id);
+		status->touch_id=g_timeout_add(STATUS_TOUCH_TIMEOUT, status_touch_timer, status);
 	}
 #ifdef ENABLE_XRECORD
 	status_record_process(status);
 #endif
+	END_FUNC
 }
 
 /* press all latched keys */
-void status_send_latched (struct status *status, struct key *key, enum status_event event)
+void status_press_latched (struct status *status, struct key *key)
 {
+	START_FUNC
 	struct key *latched;
 	GList *list=status->latched_keys;
 	while (list) {
 		latched=((struct key *)list->data);
-		status_send(status, latched, event);
+		status_press(status, latched);
 		list=list->next;
 	}
 	/* send "locked" modifier keys that are not lockers */
@@ -339,15 +309,40 @@ void status_send_latched (struct status *status, struct key *key, enum status_ev
 	while (list) {
 		latched=((struct key *)list->data);
 		if (!key_is_locker(latched)) {
-			status_send(status, latched, event);
+			status_press(status, latched);
 		}
 		list=list->next;
 	}
+	END_FUNC
+}
+
+/* release all latched keys */
+void status_release_latched (struct status *status, struct key *key)
+{
+	START_FUNC
+	struct key *latched;
+	GList *list=status->latched_keys;
+	while (list) {
+		latched=((struct key *)list->data);
+		status_release(status, latched);
+		list=list->next;
+	}
+	/* send "locked" modifier keys that are not lockers */
+	list=status->locked_keys;
+	while (list) {
+		latched=((struct key *)list->data);
+		if (!key_is_locker(latched)) {
+			status_release(status, latched);
+		}
+		list=list->next;
+	}
+	END_FUNC
 }
 
 /* calculate globalmod according to latched and locked list */
 void status_globalmod_calc(struct status *status)
 {
+	START_FUNC
 	GdkModifierType globalmod=0;
 	GList *list=status->latched_keys;
 	while (list) {
@@ -360,44 +355,52 @@ void status_globalmod_calc(struct status *status)
 		list=list->next;
 	}
 	status->globalmod=globalmod;
+	END_FUNC
 }
 
 /* latch or lock a key (depending on state) */
-void status_latchorlock (struct status *status, struct key *key, enum status_event event,
-	enum key_state state)
+void status_latchorlock (struct status *status, struct key *key, enum key_state state)
 {
+	START_FUNC
 	GList **list=(state==KEY_LATCHED?&(status->latched_keys):&(status->locked_keys));
 	*list=g_list_append(*list, key);
 	/* update globalmod */
 	status_globalmod_set(status, key_get_modifier(key));
+	END_FUNC
 }
 
 /* unlatch or unlock a key (depending on state) */
-void status_unlatchorlock (struct status *status, struct key *key,
-	enum status_event event, enum key_state state)
+void status_unlatchorlock (struct status *status, struct key *key, enum key_state state)
 {
+	START_FUNC
 	GList **list=(state==KEY_LATCHED?&(status->latched_keys):&(status->locked_keys));
 	GList *found;
 	if ((found=g_list_find(*list, key)))
 		*list=g_list_delete_link(*list, found);
 	status_globalmod_calc(status);
+	END_FUNC
 }
 
 /* latch a key */
-void status_latch (struct status *status, struct key *key, enum status_event event)
+void status_latch (struct status *status, struct key *key)
 {
-	status_latchorlock(status, key, event, KEY_LATCHED);
+	START_FUNC
+	status_latchorlock(status, key, KEY_LATCHED);
+	END_FUNC
 }
 
 /* unlatch a key */
-void status_unlatch (struct status *status, struct key *key, enum status_event event)
+void status_unlatch (struct status *status, struct key *key)
 {
-	status_unlatchorlock(status, key, event, KEY_LATCHED);
+	START_FUNC
+	status_unlatchorlock(status, key, KEY_LATCHED);
+	END_FUNC
 }
 
 /* unlatch all latched keys */
-void status_unlatch_all (struct status *status, struct key *key, enum status_event event)
+void status_unlatch_all (struct status *status, struct key *key)
 {
+	START_FUNC
 	struct key *latched;
 	while(status->latched_keys) {
 		latched=(struct key *)(g_list_first(status->latched_keys)->data);
@@ -407,24 +410,23 @@ void status_unlatch_all (struct status *status, struct key *key, enum status_eve
 		if (status->view) view_update(status->view, latched, TRUE);
 	}
 	status_globalmod_calc(status);
+	END_FUNC
 }
 
 /* lock a key*/
-void status_lock (struct status *status, struct key *key, enum status_event event)
+void status_lock (struct status *status, struct key *key)
 {
-	status_latchorlock(status, key, event, KEY_LOCKED);
+	START_FUNC
+	status_latchorlock(status, key, KEY_LOCKED);
+	END_FUNC
 }
 
 /* unlock a key */
-void status_unlock (struct status *status, struct key *key, enum status_event event)
+void status_unlock (struct status *status, struct key *key)
 {
-	status_unlatchorlock(status, key, event, KEY_LOCKED);
-}
-
-/* print a status error */
-void status_error (struct status *status, struct key *key, enum status_event event)
-{
-	flo_error(_("FSM state error. event=%d ; state=%d"), event, key->state);
+	START_FUNC
+	status_unlatchorlock(status, key, KEY_LOCKED);
+	END_FUNC
 }
 
 
@@ -435,6 +437,8 @@ struct key *status_hit_get(struct status *status, gint x, gint y, enum key_hit *
 struct key *status_hit_get(struct status *status, gint x, gint y)
 #endif
 {
+	START_FUNC
+	END_FUNC
 #ifdef ENABLE_RAMBLE
 	return view_hit_get(status->view, x, y, hit);
 #else
@@ -445,28 +449,34 @@ struct key *status_hit_get(struct status *status, gint x, gint y)
 /* start the timer */
 void status_timer_start(struct status *status, GSourceFunc update, gpointer data)
 {
+	START_FUNC
 	if (status->timer) g_timer_start(status->timer);
 	else {
 		status->timer=g_timer_new();
 		g_timeout_add(STATUS_ANIMATION_INTERVAL, update, data);
 	}
+	END_FUNC
 }
 
 /* stop the timer */
 void status_timer_stop(struct status *status)
 {
+	START_FUNC
 	if (status->timer) {
 		g_timer_destroy(status->timer);
 		status->timer=NULL;
 	}
+	END_FUNC
 }
 
 /* get timer value */
 gdouble status_timer_get(struct status *status)
 {
+	START_FUNC
 	gdouble ret=0.0;
 	if (status->timer)
-		ret=g_timer_elapsed(status->timer, NULL)*1000./settings_double_get("behaviour/timer");
+		ret=g_timer_elapsed(status->timer, NULL)*1000./settings_get_double(SETTINGS_TIMER);
+	END_FUNC
 	return ret;
 }
 
@@ -482,6 +492,7 @@ GdkModifierType status_globalmod_get(struct status *status) { return status->glo
 /* find a child window of win to focus by its name */
 struct status_focus *status_find_subwin(Window parent, const gchar *win)
 {
+	START_FUNC
 	Window root_return, parent_return;
 	Window *children;
 	guint nchildren, idx;
@@ -508,6 +519,7 @@ struct status_focus *status_find_subwin(Window parent, const gchar *win)
 	} else {
 		flo_warn(_("XQueryTree failed."));
 	}
+	END_FUNC
 	return focus;
 }
 
@@ -515,6 +527,7 @@ struct status_focus *status_find_subwin(Window parent, const gchar *win)
 /* TODO: wait for window to exist */
 struct status_focus *status_find_window(const gchar *win)
 {
+	START_FUNC
 	gchar *name;
 	struct status_focus *focus=NULL;
 	if (win && win[0]) {
@@ -534,12 +547,49 @@ struct status_focus *status_find_window(const gchar *win)
 		}
 		if (name) XFree(name);
 	}
+	END_FUNC
 	return focus;
 }
 
+/* Set input method */
+void status_im_set(struct status *status, gchar *val)
+{
+	START_FUNC
+	if (!strcmp(val, "button")) status->input_method=STATUS_IM_BUTTON;
+	else if (!strcmp(val, "timer")) status->input_method=STATUS_IM_TIMER;
+#ifdef ENABLE_RAMBLE
+	else if (!strcmp(val, "ramble")) status->input_method=STATUS_IM_RAMBLE;
+#endif
+	else if (!strcmp(val, "touch")) status->input_method=STATUS_IM_TOUCH;
+	else {
+		status->input_method=STATUS_IM_BUTTON;
+		flo_warn(_("Unknown input method: %s ; using button input method"), val);
+	}
+	END_FUNC
+}
+
+/* Called on input method change */
+void status_input_method(GSettings *settings, gchar *key, gpointer user_data)
+{
+	START_FUNC
+	struct status *status=(struct status *)user_data;
+	status_im_set(status, settings_get_string(SETTINGS_INPUT_METHOD));
+	END_FUNC
+}
+
+/* get selected input method */
+enum status_input_method status_im_get(struct status *status)
+{
+	START_FUNC
+	END_FUNC
+	return status->input_method;
+}
+
 /* allocate memory for status */
 struct status *status_new(const gchar *focus_back)
 {
+	START_FUNC
+	gchar *im;
 	struct status *status=g_malloc(sizeof(struct status));
 	if (!status) flo_fatal(_("Unable to allocate memory for status"));
 	memset(status, 0, sizeof(struct status));
@@ -551,12 +601,18 @@ struct status *status_new(const gchar *focus_back)
 	if (focus_back) {
 		status->w_focus=status_find_window(focus_back);
 	}
+	im=settings_get_string(SETTINGS_INPUT_METHOD);
+	status_im_set(status, im);
+	if (im) g_free(im);
+	settings_changecb_register(SETTINGS_INPUT_METHOD, status_input_method, status);
+	END_FUNC
 	return status;
 }
 
 /* liberate status memory */
 void status_free(struct status *status)
 {
+	START_FUNC
 #ifdef ENABLE_XRECORD
 	status_record_stop(status);
 #endif
@@ -566,11 +622,13 @@ void status_free(struct status *status)
 	if (status->locked_keys) g_list_free(status->locked_keys);
 	if (status->w_focus) g_free(status->w_focus);
 	if (status) g_free(status);
+	END_FUNC
 }
 
 /* reset the status to its original state */
 void status_reset(struct status *status)
 {
+	START_FUNC
 	status->focus=NULL;
 	status->pressed=NULL;
 	if (status->timer) g_timer_destroy(status->timer);
@@ -580,18 +638,22 @@ void status_reset(struct status *status)
 	status->latched_keys=NULL;
 	status->locked_keys=NULL;
 	status->globalmod=0;
+	END_FUNC
 }
 
 /* sets the view to update on status change */
 void status_view_set(struct status *status, struct view *view)
 {
+	START_FUNC
 	status->view=view;
 	view_status_set(view, status);
+	END_FUNC
 }
 
 /* disable sending of spi events: send xtest events instead */
 void status_spi_disable(struct status *status)
 {
+	START_FUNC
 #ifdef ENABLE_XTST
 	int event_base, error_base, major, minor;
 	if (!XTestQueryExtension(
@@ -609,13 +671,21 @@ void status_spi_disable(struct status *status)
 	flo_fatal(_("There is no way we can send keyboard events."));
 #endif
 	status->spi=FALSE;
+	END_FUNC
 }
 
 /* tell if spi is enabled */
 gboolean status_spi_is_enabled(struct status *status) { return status->spi; }
 
 /* set/get moving status */
-void status_set_moving(struct status *status, gboolean moving) { status->moving=moving; }
+void status_set_moving(struct status *status, gboolean moving)
+{
+	START_FUNC
+	status->moving=moving;
+	if (moving) gtk_window_set_gravity(view_window_get(status->view), GDK_GRAVITY_STATIC);
+	else gtk_window_set_gravity(view_window_get(status->view), GDK_GRAVITY_NORTH_WEST);
+	END_FUNC
+}
 gboolean status_get_moving(struct status *status) { return status->moving; }
 
 /* zoom the focused key */
