@@ -1,7 +1,7 @@
 /* 
  * florence - Florence is a simple virtual keyboard for Gnome.
 
- * Copyright (C) 2008, 2009, 2010 François Agrech
+ * Copyright (C) 2012 François Agrech
 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,11 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <cairo/cairo-xlib.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/extensions/shape.h>
+#include <X11/extensions/Xcomposite.h>
+
 
 /* Show the view next to the accessible object if specified. */
 #ifdef AT_SPI
@@ -40,40 +45,40 @@ void view_show (struct view *view, Accessible *object)
 void view_show (struct view *view)
 #endif
 {
-#ifndef APPLET
-#ifdef ENABLE_AT_SPI
-	/* positionnement intelligent */
-	if (settings_get_bool("behaviour/auto_hide") && 
-		settings_get_bool("behaviour/move_to_widget") && object) {
-		tools_window_move(view->window, object);
-	}
-#endif
+	START_FUNC
 	gtk_widget_show(GTK_WIDGET(view->window));
 	/* Some winwow managers forget it */
 	gtk_window_set_keep_above(view->window, TRUE);
 	/* reposition the window */
-	gtk_window_move(view->window, settings_get_int("window/xpos"), settings_get_int("window/ypos"));
-#else
-	gtk_widget_show(GTK_WIDGET(view->window));
+	gtk_window_move(view->window, settings_get_int(SETTINGS_XPOS), settings_get_int(SETTINGS_YPOS));
+#ifdef ENABLE_AT_SPI
+	/* positionnement intelligent */
+	if (settings_get_bool(SETTINGS_AUTO_HIDE) && 
+		settings_get_bool(SETTINGS_MOVE_TO_WIDGET) && object) {
+		tools_window_move(view->window, object);
+	}
 #endif
+	END_FUNC
 }
 
 /* Hides the view */
 void view_hide (struct view *view)
 {
+	START_FUNC
 	gtk_widget_hide(GTK_WIDGET(view->window));
+	END_FUNC
 }
 
 /* resize the window */
 void view_resize (struct view *view)
 {
+	START_FUNC
 	GdkRectangle rect;
 	GdkGeometry hints;
 	hints.win_gravity=GDK_GRAVITY_NORTH_WEST;
-#ifndef APPLET
-	if (settings_get_bool("window/resizable")) {
+	if (settings_get_bool(SETTINGS_RESIZABLE)) {
 		gtk_window_set_resizable(view->window, TRUE);
-		if (settings_get_bool("window/keep_ratio")) {
+		if (settings_get_bool(SETTINGS_KEEP_RATIO)) {
 			hints.min_aspect=view->vwidth/view->vheight;
 			hints.max_aspect=view->vwidth/view->vheight;
 			gtk_window_set_geometry_hints(view->window, NULL, &hints,
@@ -90,23 +95,22 @@ void view_resize (struct view *view)
 		gtk_window_set_geometry_hints(view->window, NULL, &hints,
 			GDK_HINT_WIN_GRAVITY);
 		gtk_window_set_resizable(view->window, FALSE);
-#endif
 		gtk_widget_set_size_request(GTK_WIDGET(view->window),
 			view->width, view->height);
-#ifndef APPLET
 	}
-#endif
 	/* refresh the view */
-	if (view->window && GTK_WIDGET(view->window)->window) {
+	if (view->window && gtk_widget_get_window(GTK_WIDGET(view->window))) {
 		rect.x=0; rect.y=0;
 		rect.width=view->width; rect.height=view->height;
-		gdk_window_invalidate_rect(GTK_WIDGET(view->window)->window, &rect, TRUE);
+		gdk_window_invalidate_rect(gtk_widget_get_window(GTK_WIDGET(view->window)), &rect, TRUE);
 	}
+	END_FUNC
 }
 
 /* draws the background of florence */
 void view_draw (struct view *view, cairo_t *cairoctx, cairo_surface_t **surface, enum style_class class)
 {
+	START_FUNC
 	GSList *list=view->keyboards;
 	struct keyboard *keyboard;
 	cairo_t *offscreen;
@@ -147,22 +151,29 @@ void view_draw (struct view *view, cairo_t *cairoctx, cairo_surface_t **surface,
 		list=list->next;
 	}
 	cairo_destroy(offscreen);
+	END_FUNC
 }
 
 /* draws the background of florence */
 void view_background_draw (struct view *view, cairo_t *cairoctx)
 {
+	START_FUNC
 	view_draw(view, cairoctx, &(view->background), STYLE_SHAPE);
+	END_FUNC
 }
 
 /* draws the symbols */
-void view_symbols_draw (struct view *view, cairo_t *cairoctx) {
+void view_symbols_draw (struct view *view, cairo_t *cairoctx)
+{
+	START_FUNC
 	view_draw(view, cairoctx, &(view->symbols), STYLE_SYMBOL);
+	END_FUNC
 }
 
 /* update the keyboard positions */
 void view_keyboards_set_pos(struct view *view, struct keyboard *over)
 {
+	START_FUNC
 	GSList *list=view->keyboards;
 	struct keyboard *keyboard;
 	gdouble width, height, xoffset, yoffset;
@@ -214,11 +225,13 @@ void view_keyboards_set_pos(struct view *view, struct keyboard *over)
 		}
 		list = list->next;
 	}
+	END_FUNC
 }
 
 /* calculate the dimensions of Florence */
 void view_set_dimensions(struct view *view)
 {
+	START_FUNC
 	GSList *list=view->keyboards;
 	struct keyboard *keyboard;
 	struct keyboard *over=NULL;
@@ -257,6 +270,7 @@ void view_set_dimensions(struct view *view)
 	view->width=(guint)(view->vwidth*view->scalex);
 	view->height=(guint)(view->vheight*view->scaley);
 	view_keyboards_set_pos(view, over);
+	END_FUNC
 }
 
 /* get the key at position */
@@ -266,6 +280,7 @@ struct key *view_hit_get (struct view *view, gint x, gint y, enum key_hit *hit)
 struct key *view_hit_get (struct view *view, gint x, gint y)
 #endif
 {
+	START_FUNC
 	GSList *list=view->keyboards;
 	struct keyboard *keyboard;
 	struct key *key;
@@ -292,119 +307,138 @@ struct key *view_hit_get (struct view *view, gint x, gint y)
 	key=keyboard_hit_get(keyboard, x-kx, y-ky, view->scalex, view->scaley);
 #endif
 
+	END_FUNC
 	return key;
 }
 
-#ifndef APPLET
 /* Create a window mask for transparent window for non-composited screen */
 /* For composited screen, this function is useless, use alpha channel instead. */
 void view_create_window_mask(struct view *view)
 {
-	GdkBitmap *mask=NULL;
+	START_FUNC
+	Pixmap shape;
+	cairo_surface_t *mask=NULL;
 	cairo_t *cairoctx=NULL;
+	Display *disp=(Display *)gdk_x11_get_default_xdisplay();
 
-	if (settings_get_bool("window/transparent") && (!view->composite)) {
-		if (!(mask=(GdkBitmap*)gdk_pixmap_new(NULL, view->width, view->height, 1)))
-			flo_fatal(_("Unable to create mask"));
-		cairoctx=gdk_cairo_create(mask);
+	if (settings_get_bool(SETTINGS_TRANSPARENT) && (!view->composite)) {
+		shape=XCreatePixmap(disp, GDK_WINDOW_XID(gtk_widget_get_window(GTK_WIDGET(view->window))),
+			view->width, view->height, 1);
+		mask=cairo_xlib_surface_create_for_bitmap(disp, shape,
+			DefaultScreenOfDisplay(disp), view->width, view->height);
+		cairoctx=cairo_create(mask);
 		view_background_draw(view, cairoctx);
 		cairo_set_source_rgba(cairoctx, 0.0, 0.0, 0.0, 0.0);
 		cairo_set_operator(cairoctx, CAIRO_OPERATOR_SOURCE);
 		cairo_paint(cairoctx);
 		cairo_set_source_surface(cairoctx, view->background, 0, 0);
 		cairo_paint(cairoctx);
-		gdk_window_shape_combine_mask(GTK_WIDGET(view->window)->window, mask, 0, 0);
+		XShapeCombineMask(disp, GDK_WINDOW_XID(gtk_widget_get_window(GTK_WIDGET(view->window))),
+			ShapeBounding, 0, 0, cairo_xlib_surface_get_drawable(mask), ShapeSet);
 		cairo_destroy(cairoctx);
 		cairo_surface_destroy(view->background);
 		view->background=NULL;
 		g_object_unref(G_OBJECT(mask));
 		status_focus_zoom_set(view->status, FALSE);
 	} else {
-		gdk_window_shape_combine_mask(GTK_WIDGET(view->window)->window, NULL, 0, 0);
+		XShapeCombineMask(disp, GDK_WINDOW_XID(gtk_widget_get_window(GTK_WIDGET(view->window))),
+			ShapeBounding, 0, 0, 0, ShapeSet);
 		status_focus_zoom_set(view->status, TRUE);
 	}
 	gtk_widget_queue_draw(GTK_WIDGET(view->window));
+	END_FUNC
 }
 
 /* Triggered by gconf when the "transparent" parameter is changed. Calls view_create_window_mask */
-void view_set_transparent(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_transparent(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-	gboolean shown=GTK_WIDGET_VISIBLE(GTK_WINDOW(view->window));
+	gboolean shown=gtk_widget_get_visible(GTK_WIDGET(view->window));
 	gtk_widget_show(GTK_WIDGET(view->window));
 	view_create_window_mask(view);
 	if (!shown) gtk_widget_hide(GTK_WIDGET(view->window));
+	END_FUNC
 }
 
 /* Triggered by gconf when the "decorated" parameter is changed. Decorates or undecorate the window. */
-void view_set_decorated(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_decorated(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-	gtk_window_set_decorated(view->window, gconf_value_get_bool(gconf_entry_get_value(entry)));
-	gtk_window_move(view->window, settings_get_int("window/xpos"), settings_get_int("window/ypos"));
+	gtk_window_set_decorated(view->window, settings_get_bool(SETTINGS_DECORATED));
+	gtk_window_move(view->window, settings_get_int(SETTINGS_XPOS), settings_get_int(SETTINGS_YPOS));
+	END_FUNC
 }
 
 /* Triggered by gconf when the "always_on_top" parameter is changed. 
    Change the window property to be always on top or not to be. */
-void view_set_always_on_top(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_always_on_top(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-	gtk_window_set_keep_above(view->window, gconf_value_get_bool(gconf_entry_get_value(entry)));
+	gtk_window_set_keep_above(view->window, settings_get_bool(SETTINGS_ALWAYS_ON_TOP));
+	END_FUNC
 }
 
 /* Triggered by gconf when the "task_bar" parameter is changed. 
    Change the window hint to appear in the task bar or not. */
-void view_set_task_bar(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_task_bar(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-	gtk_window_set_skip_taskbar_hint(view->window, !gconf_value_get_bool(gconf_entry_get_value(entry)));
+	gtk_window_set_skip_taskbar_hint(view->window, !settings_get_bool(SETTINGS_TASK_BAR));
+	END_FUNC
 }
 
 /* Triggered by gconf when the "resizable" parameter is changed.
    makes the window (not)resizable the window. */
-void view_set_resizable(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_resizable(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-	if (settings_get_bool("window/resizable")) {
+	if (settings_get_bool(SETTINGS_RESIZABLE)) {
 		gtk_widget_set_size_request(GTK_WIDGET(view->window), view->vwidth, view->vheight);
 	}
 	view_resize(view);
+	END_FUNC
 }
-#endif
 
 /* Triggered by gconf when a color parameter is changed. */
-void view_redraw(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_redraw(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-	char *key;
-	key=strrchr(entry->key, '/');
-	key+=1;
 	style_update_colors(view->style);
 	if ((!strcmp(key, "key")) || (!strcmp(key, "outline"))) {
 		if (view->background) cairo_surface_destroy(view->background);
 		view->background=NULL;
-	} else if (!strcmp(key, "label")) {
+	} else if (!strncmp(key, "label", 5) || (!strcmp(key, "font")) || (!strcmp(key, "system_font"))) {
 		if (view->symbols) cairo_surface_destroy(view->symbols);
 		view->symbols=NULL;
 	}
 	gtk_widget_queue_draw(GTK_WIDGET(view->window));
+	END_FUNC
 }
 
 /* Triggered by gconf when the "resizable" parameter is changed.
    makes the window (not)resizable the window. */
-void view_set_keep_ratio(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_keep_ratio(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-	if (settings_get_bool("window/keep_ratio")) {
+	if (settings_get_bool(SETTINGS_KEEP_RATIO)) {
 		view->scaley=view->scalex;
 	}
 	view_resize(view);
-	view_redraw(client, xnxn_id, entry, user_data);
+	view_redraw(settings, key, user_data);
+	END_FUNC
 }
 
 /* Redraw the key to the window */
 void view_update(struct view *view, struct key *key, gboolean statechange)
 {
+	START_FUNC
 	GdkRectangle *rect;
 	GdkCursor *cursor;
 
@@ -416,51 +450,60 @@ void view_update(struct view *view, struct key *key, gboolean statechange)
 		} else {
 			rect=keyboard_key_getrect((struct keyboard *)key_get_keyboard(key),
 				key, status_focus_zoom_get(view->status));
-			gdk_window_invalidate_rect(GTK_WIDGET(view->window)->window, rect, TRUE);
+			gdk_window_invalidate_rect(gtk_widget_get_window(GTK_WIDGET(view->window)), rect, TRUE);
 		}
 	}
 	if (status_focus_get(view->status)) {
 		if (!view->hand_cursor) {
 			cursor=gdk_cursor_new(GDK_HAND2);
-			gdk_window_set_cursor(GTK_WIDGET(view->window)->window, cursor);
+			gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(view->window)), cursor);
 			view->hand_cursor=TRUE;
 		}
 	} else if (view->hand_cursor) {
-		gdk_window_set_cursor(GTK_WIDGET(view->window)->window, NULL);
+		gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(view->window)), NULL);
 		view->hand_cursor=FALSE;
 	}
+	END_FUNC
 }
 
 /* on screen change event: check for composite extension */
 void view_screen_changed (GtkWidget *widget, GdkScreen *old_screen, struct view *view)
 {
-	GdkScreen *screen=gtk_widget_get_screen(widget);
-	GdkColormap *colormap=gdk_screen_get_rgba_colormap(screen);
-	if (colormap) {
+	START_FUNC
+	GdkVisual *visual;
+	if (gtk_widget_is_composited(widget)) {
+		flo_info(_("X11 composite extension detected. Semi-transparency is enabled."));
 		if (view) view->composite=TRUE;
+		visual=gdk_screen_get_rgba_visual(gdk_screen_get_default());
+		if (visual==NULL) visual=gdk_screen_get_system_visual(gdk_screen_get_default());
+		gtk_widget_set_visual(widget, visual);
 	} else { 
 		flo_info(_("Your screen does not support alpha channel. Semi-transparency is disabled"));
 		if (view) view->composite=FALSE;
-		colormap=gdk_screen_get_rgb_colormap(screen);
 	}
-	gtk_widget_set_colormap(widget, colormap);
+	END_FUNC
 }
 
-#ifndef APPLET
 /* on configure events: record window position */
 void view_configure (GtkWidget *window, GdkEventConfigure* pConfig, struct view *view)
 {
+	START_FUNC
 	GdkRectangle rect;
+	gint xpos, ypos;
+	if (!gtk_widget_get_visible(window)) return;
 
 	/* record window position */
-	if (settings_get_int("window/xpos")!=pConfig->x)
-		settings_set_int("window/xpos", pConfig->x);
-	if (settings_get_int("window/ypos")!=pConfig->y)
-		settings_set_int("window/ypos", pConfig->y);
+	if (gtk_window_get_decorated(GTK_WINDOW(view->window)))
+		gtk_window_get_position(GTK_WINDOW(view->window), &xpos, &ypos);
+	else { xpos=pConfig->x; ypos=pConfig->y; }
+	if (settings_get_int(SETTINGS_XPOS)!=xpos)
+		settings_set_int(SETTINGS_XPOS, xpos);
+	if (settings_get_int(SETTINGS_YPOS)!=ypos)
+		settings_set_int(SETTINGS_YPOS, ypos);
 
 	/* handle resize events */
 	if ((pConfig->width!=view->width) || (pConfig->height!=view->height)) {
-		if (settings_get_bool("window/keep_ratio")) {
+		if (settings_get_bool(SETTINGS_KEEP_RATIO)) {
 			view->scalex=view->scaley=(gdouble)pConfig->width/view->vwidth;
 		} else {
 			view->scalex=(gdouble)pConfig->width/view->vwidth;
@@ -469,8 +512,8 @@ void view_configure (GtkWidget *window, GdkEventConfigure* pConfig, struct view
 		if ((view->scalex>200.0)||(view->scaley>200.0))
 			flo_warn(_("Window size out of range :%d, %d"), view->scalex, view->scaley);
 		else {
-			settings_double_set("window/scalex", view->scalex, FALSE);
-			settings_double_set("window/scaley", view->scaley, FALSE);
+			settings_set_double(SETTINGS_SCALEX, view->scalex, FALSE);
+			settings_set_double(SETTINGS_SCALEY, view->scaley, FALSE);
 		}
 		view->width=pConfig->width; view->height=pConfig->height;
 		if (view->background) cairo_surface_destroy(view->background);
@@ -480,16 +523,19 @@ void view_configure (GtkWidget *window, GdkEventConfigure* pConfig, struct view
 		view_create_window_mask(view);
 		rect.x=0; rect.y=0;
 		rect.width=pConfig->width; rect.height=pConfig->height;
-		gdk_window_invalidate_rect(GTK_WIDGET(view->window)->window, &rect, TRUE);
+		gtk_widget_size_allocate(GTK_WIDGET(view->window), &rect);
+		gdk_window_invalidate_rect(gtk_widget_get_window(GTK_WIDGET(view->window)), &rect, TRUE);
+		gdk_window_process_updates(gtk_widget_get_window(GTK_WIDGET(view->window)), FALSE);
 	}
 
-	gdk_window_configure_finished (GTK_WIDGET(view->window)->window);
+	gdk_window_configure_finished(gtk_widget_get_window(GTK_WIDGET(view->window)));
+	END_FUNC
 }
-#endif
 
 /* draw the background of the keyboard */
 void view_draw_background (struct view *view, cairo_t *context)
 {
+	START_FUNC
 	/* prepare the background */
 	if (!view->background) {
 		view_background_draw(view, context);
@@ -499,11 +545,13 @@ void view_draw_background (struct view *view, cairo_t *context)
 	cairo_set_operator(context, CAIRO_OPERATOR_OVER);
 	cairo_set_source_surface(context, view->background, 0, 0);
 	cairo_paint(context);
+	END_FUNC
 }
 
 /* draw a list of keys (latched or locked keys) */
 void view_draw_list (struct view *view, cairo_t *context, GList *list)
 {
+	START_FUNC
 	struct keyboard *keyboard;
 	struct key *key;
 	while (list) {
@@ -512,11 +560,13 @@ void view_draw_list (struct view *view, cairo_t *context, GList *list)
 		keyboard_press_draw(keyboard, context, view->style, key, view->status);
 		list=list->next;
 	}
+	END_FUNC
 }
 
 /* draw a single key (pressed or focused) */
 void view_draw_key (struct view *view, cairo_t *context, struct key *key)
 {
+	START_FUNC
 	struct keyboard *keyboard;
 	if (key) {
 		keyboard=(struct keyboard *)key_get_keyboard(key);
@@ -525,27 +575,17 @@ void view_draw_key (struct view *view, cairo_t *context, struct key *key)
 			(gdouble)cairo_xlib_surface_get_height(view->background),
 			view->style, key, view->status);
 	}
+	END_FUNC
 }
 
-/* on expose event: draws the keyboards to the window */
-void view_expose (GtkWidget *window, GdkEventExpose* pExpose, struct view *view)
+/* on draw event: draws the keyboards to the window */
+void view_expose (GtkWidget *window, cairo_t* context, struct view *view)
 {
-	cairo_t *context;
-
-	/* Don't need to redraw several times in one chunk */
-	if (!view->redraw) view->redraw=gdk_region_new();
-	gdk_region_union(view->redraw, pExpose->region);
-	if (pExpose->count>0) return;
-
-	/* create the context */
-	context=gdk_cairo_create(window->window);
-	gdk_cairo_region(context, view->redraw);
-	cairo_clip(context); 
-	gdk_region_destroy(view->redraw);
-	view->redraw=NULL;
+	START_FUNC
+	enum key_state state;
 
 	/* clear the area */
-	if (settings_get_bool("window/transparent")) {
+	if (settings_get_bool(SETTINGS_TRANSPARENT)) {
 		cairo_set_source_rgba(context, 0.0, 0.0, 0.0, 0.0);
 		cairo_set_operator(context, CAIRO_OPERATOR_SOURCE);
 		cairo_paint(context);
@@ -562,14 +602,14 @@ void view_expose (GtkWidget *window, GdkEventExpose* pExpose, struct view *view)
 
 	/* handle composited transparency */
 	/* TODO: check for transparency support in WM */
-	if (view->composite && settings_double_get("window/opacity")!=100.0) {
-		if (settings_double_get("window/opacity")>100.0 ||
-			settings_double_get("window/opacity")<1.0) {
+	if (view->composite && settings_get_double(SETTINGS_OPACITY)!=100.0) {
+		if (settings_get_double(SETTINGS_OPACITY)>100.0 ||
+			settings_get_double(SETTINGS_OPACITY)<1.0) {
 			flo_error(_("Window opacity out of range (1.0 to 100.0): %f"),
-				settings_double_get("window/opacity"));
+				settings_get_double(SETTINGS_OPACITY));
 		}
 		cairo_set_source_rgba(context, 0.0, 0.0, 0.0,
-			(100.0-settings_double_get("window/opacity"))/100.0);
+			(100.0-settings_get_double(SETTINGS_OPACITY))/100.0);
 		cairo_set_operator(context, CAIRO_OPERATOR_DEST_OUT);
 		cairo_paint(context);
 		cairo_set_operator(context, CAIRO_OPERATOR_OVER);
@@ -583,8 +623,13 @@ void view_expose (GtkWidget *window, GdkEventExpose* pExpose, struct view *view)
 	view_draw_list(view, context, status_list_locked(view->status));
 
 	/* pressed and focused key */
-	view_draw_key(view, context, status_pressed_get(view->status));
 	view_draw_key(view, context, status_focus_get(view->status));
+	if (status_pressed_get(view->status)) {
+		state=status_pressed_get(view->status)->state;
+		key_state_set(status_pressed_get(view->status), KEY_PRESSED);
+		view_draw_key(view, context, status_pressed_get(view->status));
+		key_state_set(status_pressed_get(view->status), state);
+	}
 
 	cairo_restore(context);
 
@@ -592,38 +637,48 @@ void view_expose (GtkWidget *window, GdkEventExpose* pExpose, struct view *view)
 	if (view->ramble) ramble_draw(view->ramble, context);
 #endif
 
-	/* and free up drawing memory */
-	cairo_destroy(context);
-
-#ifndef APPLET
 	/* restore configure event handler. */
 	if (!view->configure_handler) 
 		view->configure_handler=g_signal_connect(G_OBJECT(view->window), "configure-event",
 			G_CALLBACK(view_configure), view);
-#endif
+	END_FUNC
 }
 
 /* on keys changed events */
 void view_on_keys_changed(gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
 	if (view->symbols) cairo_surface_destroy(view->symbols);
 	view->symbols=NULL;
 	gtk_widget_queue_draw(GTK_WIDGET(view->window));
+	END_FUNC
 }
 
-/* Triggered by gconf when the "extensions" parameter is changed. */
-void view_update_extensions(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+/* track the windows state changes */
+void view_window_state (GtkWidget *window, GdkEventWindowState *event, struct view *view)
 {
+	START_FUNC
+	gint is_iconified=gdk_window_get_state(gtk_widget_get_window(window))&GDK_WINDOW_STATE_ICONIFIED;
+	if (is_iconified) {
+		view_hide(view);
+		gtk_window_deiconify(GTK_WINDOW(view->window));
+	} 
+	END_FUNC
+}
+
+
+/* Triggered by gconf when the "extensions" parameter is changed. */
+void view_update_extensions(GSettings *settings, gchar *key, gpointer user_data)
+{
+	START_FUNC
 	struct view *view=(struct view *)user_data;
 	GSList *list=view->keyboards;
 	struct keyboard *keyboard;
 
-#ifndef APPLET
 	/* Do not call configure signal handler */
 	if (view->configure_handler) g_signal_handler_disconnect(G_OBJECT(view->window), view->configure_handler);
 	view->configure_handler=0;
-#endif
 
 	while (list)
 	{
@@ -638,79 +693,79 @@ void view_update_extensions(GConfClient *client, guint xnxn_id, GConfEntry *entr
 	view->background=NULL;
 	if (view->symbols) cairo_surface_destroy(view->symbols);
 	view->symbols=NULL;
-#ifndef APPLET
 	view_create_window_mask(view);
-#endif
 	status_focus_set(view->status, NULL);
 	gtk_widget_queue_draw(GTK_WIDGET(view->window));
+	END_FUNC
 }
 
 /* Triggered by gconf when the "zoom" parameter is changed. */
-void view_set_scalex(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_scalex(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-#ifndef APPLET
 	/* Do not call configure signal handler */
 	if (view->configure_handler) g_signal_handler_disconnect(G_OBJECT(view->window), view->configure_handler);
 	view->configure_handler=0;
-#endif
-	view->scalex=gconf_value_get_float(gconf_entry_get_value(entry));
-	if (settings_get_bool("window/keep_ratio")) view->scaley=view->scalex;
-	view_update_extensions(client, xnxn_id, entry, user_data);
+	view->scalex=settings_get_double(SETTINGS_SCALEX);
+	if (settings_get_bool(SETTINGS_KEEP_RATIO)) view->scaley=view->scalex;
+	view_update_extensions(settings, key, user_data);
+	END_FUNC
 }
 
 /* Triggered by gconf when the "zoom" parameter is changed. */
-void view_set_scaley(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_scaley(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
-#ifndef APPLET
 	/* Do not call configure signal handler */
 	if (view->configure_handler) g_signal_handler_disconnect(G_OBJECT(view->window), view->configure_handler);
 	view->configure_handler=0;
-#endif
-	view->scaley=gconf_value_get_float(gconf_entry_get_value(entry));
-	if (settings_get_bool("window/keep_ratio")) view->scalex=view->scaley;
-	view_update_extensions(client, xnxn_id, entry, user_data);
+	view->scaley=settings_get_double(SETTINGS_SCALEY);
+	if (settings_get_bool(SETTINGS_KEEP_RATIO)) view->scalex=view->scaley;
+	view_update_extensions(settings, key, user_data);
+	END_FUNC
 }
 
 /* Triggered by gconf when the "opacity" parameter is changed. */
-void view_set_opacity(GConfClient *client, guint xnxn_id, GConfEntry *entry, gpointer user_data)
+void view_set_opacity(GSettings *settings, gchar *key, gpointer user_data)
 {
+	START_FUNC
 	struct view *view=(struct view *)user_data;
 	gtk_widget_queue_draw(GTK_WIDGET(view->window));
+	END_FUNC
 }
 
 /* get gtk window of the view */
-#ifdef APPLET
-PanelApplet *view_window_get (struct view *view)
-#else
 GtkWindow *view_window_get (struct view *view)
-#endif
 {
+	START_FUNC
+	END_FUNC
 	return view->window;
 }
 
 /* get gtk window of the view */
 void view_status_set (struct view *view, struct status *status)
 {
+	START_FUNC
 	view->status=status;
+	END_FUNC
 }
 
 /* liberate all the memory used by the view */
 void view_free(struct view *view)
 {
+	START_FUNC
 	if (view->background) cairo_surface_destroy(view->background);
 	if (view->symbols) cairo_surface_destroy(view->symbols);
 	g_free(view);
+	END_FUNC
 }
 
 /* create a view of florence */
-#ifdef APPLET
-struct view *view_new (struct status *status, struct style *style, GSList *keyboards, PanelApplet *applet)
-#else
 struct view *view_new (struct status *status, struct style *style, GSList *keyboards)
-#endif
 {
+	START_FUNC
 	struct view *view=g_malloc(sizeof(struct view));
 	if (!view) flo_fatal(_("Unable to allocate memory for view"));
 	memset(view, 0, sizeof(struct view));
@@ -718,83 +773,67 @@ struct view *view_new (struct status *status, struct style *style, GSList *keybo
 	view->status=status;
 	view->style=style;
 	view->keyboards=keyboards;
-	view->scalex=settings_double_get("window/scalex");
-	view->scaley=settings_double_get("window/scaley");
+	view->scalex=settings_get_double(SETTINGS_SCALEX);
+	view->scaley=settings_get_double(SETTINGS_SCALEY);
 	view_set_dimensions(view);
-#ifdef APPLET
-	view->window = applet;
-#else
 	view->window=GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
-	gtk_window_set_keep_above(view->window, settings_get_bool("window/always_on_top"));
+	gtk_window_set_keep_above(view->window, settings_get_bool(SETTINGS_ALWAYS_ON_TOP));
  	gtk_window_set_accept_focus(view->window, FALSE);
-	gtk_window_set_skip_taskbar_hint(view->window, !settings_get_bool("window/task_bar"));
-#ifdef HAVE_RESIZE_GRIP
+	gtk_window_set_skip_taskbar_hint(view->window, !settings_get_bool(SETTINGS_TASK_BAR));
 	/* Remove resize grip since it is buggy */
 	gtk_window_set_has_resize_grip(view->window, FALSE);
-#endif
-#endif
 	view_resize(view);
 	gtk_container_set_border_width(GTK_CONTAINER(view->window), 0);
-#if GTK_CHECK_VERSION(2,12,0)
 	gtk_widget_set_events(GTK_WIDGET(view->window),
 		GDK_EXPOSURE_MASK|GDK_POINTER_MOTION_HINT_MASK|GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK|
 		GDK_ENTER_NOTIFY_MASK|GDK_LEAVE_NOTIFY_MASK|GDK_STRUCTURE_MASK|GDK_POINTER_MOTION_MASK);
-#else
-	gtk_widget_set_events(GTK_WIDGET(view->window),
-		GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK|
-		GDK_ENTER_NOTIFY_MASK|GDK_LEAVE_NOTIFY_MASK|GDK_STRUCTURE_MASK|GDK_POINTER_MOTION_MASK);
-#endif
 	gtk_widget_set_app_paintable(GTK_WIDGET(view->window), TRUE);
-#ifndef APPLET
-	gtk_window_set_decorated(view->window, settings_get_bool("window/decorated"));
-	gtk_window_move(view->window, settings_get_int("window/xpos"), settings_get_int("window/ypos"));
-#endif
+	gtk_window_set_decorated(view->window, settings_get_bool(SETTINGS_DECORATED));
+	gtk_window_move(view->window, settings_get_int(SETTINGS_XPOS), settings_get_int(SETTINGS_YPOS));
 	/*g_signal_connect(gdk_keymap_get_default(), "keys-changed", G_CALLBACK(view_on_keys_changed), view);*/
 	xkeyboard_register_events(status->xkeyboard, view_on_keys_changed, (gpointer)view);
 	g_signal_connect(G_OBJECT(view->window), "screen-changed", G_CALLBACK(view_screen_changed), view);
-#ifndef APPLET
 	view->configure_handler=g_signal_connect(G_OBJECT(view->window), "configure-event",
 		G_CALLBACK(view_configure), view);
-#endif
-	g_signal_connect(G_OBJECT(view->window), "expose-event", G_CALLBACK(view_expose), view);
+	g_signal_connect(G_OBJECT(view->window), "draw", G_CALLBACK(view_expose), view);
+	g_signal_connect(G_OBJECT(view->window), "window-state-event", G_CALLBACK(view_window_state), view);
 	view_screen_changed(GTK_WIDGET(view->window), NULL, view);
 	gtk_widget_show(GTK_WIDGET(view->window));
-#ifndef APPLET
 	view_create_window_mask(view);
-#endif
 
 	/* register settings callbacks */
-#ifndef APPLET
-	settings_changecb_register("window/transparent", view_set_transparent, view);
-	settings_changecb_register("window/decorated", view_set_decorated, view);
-	settings_changecb_register("window/resizable", view_set_resizable, view);
-	settings_changecb_register("window/always_on_top", view_set_always_on_top, view);
-	settings_changecb_register("window/task_bar", view_set_task_bar, view);
-#endif
-	settings_changecb_register("window/keep_ratio", view_set_keep_ratio, view);
-	settings_changecb_register("window/scalex", view_set_scalex, view);
-	settings_changecb_register("window/scaley", view_set_scaley, view);
-	settings_changecb_register("window/opacity", view_set_opacity, view);
-	settings_changecb_register("layout/extensions", view_update_extensions, view);
-	settings_changecb_register("colours/key", view_redraw, view);
-	settings_changecb_register("colours/outline", view_redraw, view);
-	settings_changecb_register("colours/label", view_redraw, view);
-	settings_changecb_register("colours/label_outline", view_redraw, view);
-	settings_changecb_register("colours/activated", view_redraw, view);
-	settings_changecb_register("colours/latched", view_redraw, view);
+	settings_changecb_register(SETTINGS_TRANSPARENT, view_set_transparent, view);
+	settings_changecb_register(SETTINGS_DECORATED, view_set_decorated, view);
+	settings_changecb_register(SETTINGS_RESIZABLE, view_set_resizable, view);
+	settings_changecb_register(SETTINGS_ALWAYS_ON_TOP, view_set_always_on_top, view);
+	settings_changecb_register(SETTINGS_TASK_BAR, view_set_task_bar, view);
+	settings_changecb_register(SETTINGS_KEEP_RATIO, view_set_keep_ratio, view);
+	settings_changecb_register(SETTINGS_SCALEX, view_set_scalex, view);
+	settings_changecb_register(SETTINGS_SCALEY, view_set_scaley, view);
+	settings_changecb_register(SETTINGS_OPACITY, view_set_opacity, view);
+	settings_changecb_register(SETTINGS_EXTENSIONS, view_update_extensions, view);
+	settings_changecb_register(SETTINGS_KEY, view_redraw, view);
+	settings_changecb_register(SETTINGS_OUTLINE, view_redraw, view);
+	settings_changecb_register(SETTINGS_LABEL, view_redraw, view);
+	settings_changecb_register(SETTINGS_LABEL_OUTLINE, view_redraw, view);
+	settings_changecb_register(SETTINGS_ACTIVATED, view_redraw, view);
+	settings_changecb_register(SETTINGS_LATCHED, view_redraw, view);
+	settings_changecb_register(SETTINGS_SYSTEM_FONT, view_redraw, view);
+	settings_changecb_register(SETTINGS_FONT, view_redraw, view);
 
-#ifndef APPLET
 	/* set the window icon */
 	tools_set_icon(view->window);
-#endif
+	END_FUNC
 	return view;
 }
 
 /* Change the layout and style of the view and redraw */
 void view_update_layout(struct view *view, struct style *style, GSList *keyboards)
 {
+	START_FUNC
 	view->style=style;
 	view->keyboards=keyboards;
-	view_update_extensions(NULL, 0, NULL, (gpointer)view);
+	view_update_extensions(NULL, NULL, (gpointer)view);
+	END_FUNC
 }
 
